use rocket::{fairing::{Fairing, Info, Kind}, fs::FileServer, futures::io::Cursor, get, http::{ContentType, CookieJar, Header, HeaderMap, Status}, outcome::Outcome, request::{self, FromRequest}, response::content, routes, Error, Request, Response, State};
use s3::S3;
use tracing::{debug, info, trace, Level};
use tracing_subscriber::EnvFilter;

mod s3;

struct Config<'a> {
    s3_url: &'a str,
    s3_bucket: &'a str,
    s3_access_key: &'a str,
    s3_secret_key: &'a str,
}

#[rocket::main]
async fn main() {
    tracing_subscriber::fmt()
        .with_line_number(false)
        .without_time()
        .with_env_filter(EnvFilter::from_default_env())
        .init();

    let config = Config {
        s3_bucket: "b64v2",
        s3_url: "http://localhost:9000",
        s3_access_key: "Eyp21VVclXP8xx49bJ6D",
        s3_secret_key: "VxqU9hQ2gl7TvHIFzCM74Og6rQUZcpgYpHNUBnVm",
    };

    let s3 = S3::connect(&config).await.expect("Failed to connect to minio, aborting.");
    // let cors = CORS;

    let _ = rocket::build()
        .mount("/", FileServer::new("www/"))
        .mount("/", routes![get_s3_content])
        .manage(s3)
        // .attach(cors)
        .launch()
        .await;

    info!("Bye");
}

struct Og {
    og: Option<String>
}
#[rocket::async_trait]
impl<'r> FromRequest<'r> for Og{
    type Error = Error;

    async fn from_request(req: &'r Request<'_>) -> request::Outcome<Self, Self::Error> {
        if let Some(h) = req.headers().get("og-domain").next() {
            debug!(h);
            return Outcome::Success(Og { og: Some(h.to_string()) });
        }
        Outcome::Success(Og { og: None })
    }
}


#[get("/s3/<domain>/<file>")]
async fn get_s3_content(domain: &str, file: &str, db: &State<S3>, og: Og) -> (Status, (ContentType, String)) {
    use base64::prelude::*;
    let filename = domain.to_owned() + "/" + file; 

    info!("{:?}", og.og);
    
    if let Some(resp) = db.get(&filename).await {
        let content_type = if let Ok(url) = BASE64_URL_SAFE.decode(file) {
            let url = url.into_iter().map(|f| f as char).collect::<String>();
            if let Some(filetype) = url.split('.').collect::<Vec<&str>>().last() {
                ContentType::from_extension(&filetype).unwrap_or(ContentType::HTML)
            } else {
                ContentType::HTML
            }
        } else {
            ContentType::Any
        };
        trace!("{:?}", content_type);

        return (Status::Ok, (content_type, resp))
    }
    // instead of some/none I would rather this be 200/404
    // TODO should tell surreal to get this url next.
    (Status::NotFound, (ContentType::Text, "Resource not found in offline cache".to_string()))
}

// CORS, to allow other sites to request this from their front-end
// pub struct CORS;

// #[rocket::async_trait]
// impl Fairing for CORS {
//     fn info(&self) -> Info {
//         Info {
//             name: "Add CORS headers to responses",
//             kind: Kind::Response
//         }
//     }

//     async fn on_response<'r>(&self, _request: &'r Request<'_>, response: &mut Response<'r>) {
//         response.set_header(Header::new("Access-Control-Allow-Origin", "*"));
//         response.set_header(Header::new("Access-Control-Allow-Methods", "POST, GET, PATCH, OPTIONS"));
//         response.set_header(Header::new("Access-Control-Allow-Headers", "*"));
//         response.set_header(Header::new("Access-Control-Allow-Credentials", "true"));
//         // TODO set this automatically like how Rocket/core/lib/src/fs/server.rs does (bottom of file)
//         response.set_header(Header::new("Content-Type", "text/html"));
//     }
// }
